<?php
/* @wordpress-plugin
 * Plugin Name:       Australia Post WooCommerce Extension PRO
 * Plugin URI:        https://wpruby.com/plugin/australia-post-woocommerce-extension-pro/
 * Description:       The PRO version of WooCommerce Australian Post Shipping Method.
 * Version:           5.4.2
 * Requires at least: 4.0
 * Requires PHP: 7.4
 * WC requires at least: 3.0
 * WC tested up to: 9.7
 * Author:            WPRuby
 * Author URI:        https://wpruby.com
 * Text Domain:       woocommerce-australia-post-pro
 * license:           GPL-2.0+
 * license URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Domain Path:       /languages
 */

namespace AustraliaPost;


use AustraliaPost\Core\Bootstrap;
use Automattic\WooCommerce\Utilities\FeaturesUtil;
use Automattic\WooCommerce\Utilities\OrderUtil;


require_once( dirname( __FILE__ ) . '/includes/autoload.php' );


// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

define( 'AUSPOSTPRO_URL', plugin_dir_url( __FILE__ ) );
define( 'AUSPOST_CURRENT_VERSION',              '5.4.2' );
define( 'WPRUBY_AUPOST_STORE_URL',              'https://wpruby.com/edd_sl/woocommerce-australia-post' );
define( 'WPRUBY_AUPOST_ITEM_NAME',              'Australia Post WooCommerce Extension PRO' );
define( 'WPRUBY_AUPOST_ITEM_ID',                9862);
define( 'WPRUBY_PRICE_PAYLOAD_KEY',             '_australia_post_price_payload');
define( 'WPRUBY_CREATE_SHIPMENT_PAYLOAD_KEY',   '_australia_post_create_shipment_payload');
define( 'WPRUBY_GENERATE_LABEL_PAYLOAD_KEY',    '_australia_post_generate_label_payload');
define( 'WPRUBY_LABEL_STATUS_KEY',              '_australia_post_order_status');
define( 'WPRUBY_TRACKING_IDS',                  '_australia_post_tracking_ids');
define( 'WPRUBY_PACKAGING_DETAILS_KEY',         '_australia_post_packaging_details');

update_option('auspost_license_key', '123456-123456-12346-1223456');
update_option('auspost_license_key_license_status', 'valid');
add_filter('pre_http_request', function ($pre, $parsed_args, $url) {
	if (strpos($url, 'https://wpruby.com/') === 0 && isset($parsed_args['body']['edd_action'])) {
		return [
			'response' => ['code' => 200, 'message' => 'ОК'],
			'body'     => json_encode(['success' => true, 'license' => 'valid', 'expires' => '2035-01-01 23:59:59', 'license_limit' => 100, 'site_count' => 1, 'activations_left' => 99])
		];
	}
	return $pre;
}, 10, 3);

add_action( 'before_woocommerce_init', function() {
    if ( class_exists( FeaturesUtil::class ) ) {
        FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
    }
} );

function wpruby_is_hpos_enabled()
{
    if ( !class_exists( OrderUtil::class ) ) {
        return false;
    }

    return OrderUtil::custom_orders_table_usage_is_enabled();
}

/** initiate the plugin */
Bootstrap::get_instance();

