<?php

namespace AustraliaPost\BoxPacker;

use AustraliaPost\Deps\DVDoug\BoxPacker\Box;

class Australia_Post_Box implements Box
{
    private $description;
    private $OuterWidth;
    private $OuterLength;
    private $OuterDepth;
    private $EmptyWeight;
    private $InnerWidth;
    private $InnerLength;
    private $InnerDepth;
    private $MaxWeight;
    private $isSatchel = false;
    private $isFake = false;

    public function __construct(
    	$description,
	    $OuterLength,
	    $OuterWidth,
	    $OuterDepth,
	    $EmptyWeight,
	    $InnerLength,
	    $InnerWidth,
	    $InnerDepth,
	    $MaxWeight
    )
    {
        $this->description = $description;
        $this->OuterWidth = intval($OuterWidth);
        $this->OuterLength = intval($OuterLength);
        $this->OuterDepth = intval($OuterDepth);
        $this->EmptyWeight = intval($EmptyWeight);
        $this->InnerWidth = intval($InnerWidth);
        $this->InnerLength = intval($InnerLength);
        $this->InnerDepth = intval($InnerDepth);
        $this->MaxWeight = intval($MaxWeight);
    }

    /**
     * Reference for box type (e.g. SKU or description)
     * @return string
     */
    public function getReference(): string
    {
        return $this->description;
    }

    /**
     * Outer width in mm
     * @return int
     */
    public function getOuterWidth(): int
    {
        return $this->OuterWidth;
    }

    /**
     * Outer length in mm
     * @return int
     */
    public function getOuterLength(): int
    {
        return $this->OuterLength;
    }

    /**
     * Outer depth in mm
     * @return int
     */
    public function getOuterDepth(): int
    {
        return $this->OuterDepth;
    }

    /**
     * Empty weight in g
     * @return int
     */
    public function getEmptyWeight(): int
    {
        return $this->EmptyWeight;
    }

    /**
     * Inner width in mm
     * @return int
     */
    public function getInnerWidth(): int
    {
        return $this->InnerWidth;
    }

    /**
     * Inner length in mm
     * @return int
     */
    public function getInnerLength(): int
    {
        return $this->InnerLength;
    }

    /**
     * Inner depth in mm
     * @return int
     */
    public function getInnerDepth(): int
    {
        return $this->InnerDepth;
    }

    /**
     * Total inner volume of packing in mm^3
     * @return int
     */
    public function getInnerVolume()
    {
        $innerWidth = intval($this->InnerWidth) === 0? 1: $this->InnerWidth;
        $innerLength = intval($this->InnerLength) === 0? 1: $this->InnerLength;
        $innerDepth = intval($this->InnerDepth) === 0? 1: $this->InnerDepth;
        return $innerWidth * $innerLength * $innerDepth;
    }

    /**
     * Max weight the packaging can hold in g
     * @return int
     */
    public function getMaxWeight(): int
    {
        return $this->MaxWeight;
    }

	/**
	 * @return boolean
	 */
	public function isSatchel()
	{
		return $this->isSatchel;
	}

	/**
	 * @param boolean $isSatchel
	 *
	 * @return Australia_Post_Box
	 */
	public function setIsSatchel( $isSatchel ) {
		$this->isSatchel = $isSatchel;

		return $this;
	}

	/**
	 * @return boolean
	 */
	public function isFake(): bool
	{
		return $this->isFake;
	}

	/**
	 * @param boolean $isFake
	 *
	 * @return Australia_Post_Box
	 */
	public function setIsFake( bool $isFake ): self {
		$this->isFake = $isFake;

		return $this;
	}

	public function toArray(): array
	{
		return [
			'length' => $this->getOuterLength(),
			'width' => $this->getOuterWidth(),
			'depth' => $this->getOuterDepth(),
			'max_weight' => $this->getMaxWeight(),
		];
	}


}
