<?php
namespace AustraliaPost\BoxPacker;

use AustraliaPost\Deps\DVDoug\BoxPacker\Box;
use AustraliaPost\Deps\DVDoug\BoxPacker\ConstrainedPlacementItem;
use AustraliaPost\Deps\DVDoug\BoxPacker\PackedItemList;


class Australia_Post_Item  implements ConstrainedPlacementItem
{
    public $description;
    public $width;
    public $length;
    public $depth;
    public $weight;
    public $volume;
    public $keepFlat;
    public $postcode;
    public $value;
    private $product_id;

    public function __construct($description, $width, $length, $depth, $weight, $keepFlat = false, $postcode = '', $value = 0)
    {
        $this->description = $description;
        $this->width = $width;
        $this->length = $length;
        $this->depth = $depth;
        $this->weight = intval($weight);
        $this->keepFlat = $keepFlat;
        $this->postcode = $postcode;
        $this->value = $value;
        $this->volume = $this->length * $this->width * $this->depth;
    }

	/**
	 * @return mixed
	 */
	public function getPostcode()
    {
    	return $this->postcode;
    }

    /**
    * Item SKU etc
    * @return string
    */
    public function getDescription(): string
    {
        return $this->description;
    }

    /**
     * Item width in mm
     * @return int
     */
    public function getWidth(): int
    {
        return $this->width;
    }

    /**
     * Item length in mm
     * @return int
     */
    public function getLength(): int
    {
        return $this->length;
    }

    /**
     * Item depth in mm
     * @return int
     */
    public function getDepth(): int
    {
        return $this->depth;
    }

    /**
     * Item weight in g
     * @return int
     */
    public function getWeight(): int
    {
        return $this->weight;
    }

    /**
     * Item volume in mm^3
     * @return int
     */
    public function getVolume()
    {
        return $this->length * $this->width * $this->depth;
    }

    /**
     * Does this item need to be kept flat?
     * XXX not yet used, all items are kept flat
     * @return bool
     */
    public function getKeepFlat(): bool
    {
        return $this->keepFlat;
    }

	public function getValue()
	{
		return $this->value;
	}

	/**
	 * @param Box $box
	 * @param PackedItemList $alreadyPackedItems
	 * @param int $proposedX
	 * @param int $proposedY
	 * @param int $proposedZ
	 * @param int $width
	 * @param int $length
	 * @param int $depth
	 *
	 * @return bool
	 */
	public function canBePacked(Box $box, PackedItemList $alreadyPackedItems, int $proposedX, int $proposedY, int $proposedZ, int $width, int $length, int $depth): bool
	{
        $alreadyPackedItemsForHook = clone $alreadyPackedItems;
		foreach ($alreadyPackedItems as $packedItem) {
			if ($packedItem->getItem()->getPostcode() !== $this->getPostcode()  ) {
				return false;
			}
		}

		return apply_filters('australia_post_can_be_packed', true, $box, $alreadyPackedItemsForHook, $proposedX, $proposedY, $proposedZ, $width, $length, $depth, $this->getProductId() );
	}

    /**
     * @return mixed
     */
    public function getProductId()
    {
        return $this->product_id;
    }

    /**
     * @param mixed $product_id
     */
    public function setProductId($product_id)
    {
        $this->product_id = $product_id;
    }
}
